import string
import random
from django.db import models
from django.utils.translation import gettext_lazy as _

from parler.models import TranslatableModel, TranslatedFields
from manufacturers.models import Manufacturer
from brands.models import Brand
from params.models import Param


class Car(TranslatableModel):
    name = models.CharField(_("name"), max_length=128, null=True, blank=False)
    code = models.CharField(_("code"), max_length=128, null=True, blank=True, unique=True)
    translations = TranslatedFields(
        description=models.CharField(_("description"), max_length=256, null=True, blank=True),
    )
    price = models.DecimalField(_("price"), max_digits=10, decimal_places=2)
    created_dt = models.DateTimeField(_("created at"))
    brand = models.ForeignKey(
        Brand, verbose_name=_("brand"), related_name="cars", on_delete=models.SET_NULL, null=True, blank=False)
    params = models.ManyToManyField(Param, verbose_name=_("parameters"), blank=False)

    class Meta:
        verbose_name = _("car")
        verbose_name_plural = _("cars")

    def __str__(self):
        return f"{self.brand} - {self.name}"

    @staticmethod
    def generate_random_code(size=8, chars=string.ascii_uppercase + string.digits):
        return ''.join(random.choice(chars) for _ in range(size))

    def save(self, *args, **kwargs):
        self.code = Car.generate_random_code()

        super().save(*args, **kwargs)