import os
import sys
import django

SETTINGS = 'elasticsearch_cars.settings'
sys.path.append(os.path.abspath(os.path.join(os.path.realpath(__file__), "..", "..")))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", SETTINGS)
django.setup()

from datetime import datetime
from search.analyzers import *
from elasticsearch_dsl import Index
from elasticsearch_dsl.connections import Elasticsearch
from elasticsearch_dsl.query import Q
from elasticsearch_dsl.aggs import A
from examples.utils import print_response, pretty_json, print_prepared_query
from search.documents import CarDocument
from cars.models import Car

# 1] metric aggregation - total price of all cars
#    - using `A` object
#    - NOTE: [0:0] to filter out normal hits since we are aiming here for aggregations
#    - NOTE: aggregation query are not put back to search object!

# agg_query = A("sum", field="price")
# search = CarDocument.search()[0:0]
# search.aggs.metric("total_price", agg_query)
# response = search.execute()
# print_response(response, fields=["total_price"])

# 2] bucket aggregation - all car brands
#    - see .raw, which change Text into Keyword

# agg_query = A("terms", field="brand_name.raw")
# search = CarDocument.search()[0:0]
# search.aggs.bucket("car_brands", agg_query)
# response = search.execute()
# print_response(response, fields=["car_brands"])


# 3] metric and pipeline aggregation - compute max price from each bucket
#    split into two buckets - to 800000 and from 800001 and also buckets are
#    sorted based on max price from each bucket
#   - using bucket, metric and pipeline aggregations
#   - NOTE: bucket is parent aggregation, metric and pipeline nested aggregations

# agg_query_bucket = A("range", field="price", ranges=[{"to": "800000"}, {"from": "800001"}])
# agg_query_metric = A("max", field="price")
# agg_query_pipeline = A("bucket_sort", sort=[{"max_price": "asc"}])
# search = CarDocument.search()[0:0]
# search.aggs.bucket("total_price", agg_query_bucket) \
#                 .metric("max_price", agg_query_metric) \
#                 .pipeline("the_bucket_sort", agg_query_pipeline)
# print_prepared_query(search)
# response = search.execute()
# print_response(response, fields=["total_price"])
