import os
import sys
import django

SETTINGS = 'elasticsearch_cars.settings'
sys.path.append(os.path.abspath(os.path.join(os.path.realpath(__file__), "..", "..")))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", SETTINGS)
django.setup()

from datetime import datetime
from search.analyzers import *
from elasticsearch_dsl import Index
from elasticsearch_dsl.connections import Elasticsearch
from elasticsearch_dsl.query import Q
from examples.utils import print_response, print_prepared_query
from search.documents import CarDocument
from cars.models import Car

# 1] filtering cars by code
#  - NOTE: no score, because we are in the filter context
# filter_query = Q("term", code="G0N47WCY")
# response = CarDocument.search().filter(filter_query).execute()
# print_response(response, fields=["name", "code", "score"])

# 2] filtering cars by wrong code
#    - using terms == same thing as term but array support
#    - NOTE: first code does not exists, second does
# filter_query = Q("terms", code=["6ZRHJLMOXXX", "G0N47WCY"])
# response = CarDocument.search().filter(filter_query).execute()
# print_response(response, fields=["name", "code"])

# 3] filtering cars with price from 500000 to 950000
#    - using range query
#    - NOTE: this cannot be write like this: Q("range", price__lte=950000, price__gte=500000)
# filter_query = Q("range", price={"lte": 950000, "gte": 500000})
# response = CarDocument.search().filter(filter_query).execute()
# print_response(response, fields=["brand_name", "name", "price"])

# 4] filtering cars with price greater than 1 000 000 and created in the year 2019
#    - using range query
#    - using queries combining
#    - we use date format and date rounding (otherwise it does not search only in year 2019 correctly)
# filter_query = Q("range", price={"gte": 1000000}) & Q("range", created_dt={"lte": "2019||/y", "gte": "2019||/y", "format": "yyyy"})
# response = CarDocument.search().filter(filter_query).execute()
# print_response(response, fields=["name", "price", "created_dt"])

# 5] filtering cars by parameters - all BMW's or Mercedes with top speed higher than 200 and petrol engine
#    - using nested filter
#    - using queries combining
#    - NOTE: we must specify `nested` and `path` arguments in `.filter()` function
#    - NOTE: parameters name ane value must be written like: `<path>__name`, `<path>__value`
#    - NOTE: it must be as three separate nested filter queries, not one big, because in that case we have
#            params__name="brand" & params__name="top_speed" -> elastic does not now which one use
# filter_query_1 = Q("term", params__name="brand") & (Q("term", params__value="bmw") | Q("term", params__value="mercedes"))
# filter_query_2 = (Q("term", params__name="top-speed") & Q("range", params__value={"gte": 200}))
# filter_query_3 = (Q("term", params__name="engine-type") & Q("term", params__value="petrol"))
#
# search = CarDocument.search() \
#             .filter("nested", path="params", query=filter_query_1) \
#             .filter("nested", path="params", query=filter_query_2) \
#             .filter("nested", path="params", query=filter_query_3)
#
# # print_prepared_query(search)
# response = search.execute()
# print_response(response, fields=["name", "params"])
