import os
import sys
import django

SETTINGS = 'elasticsearch_cars.settings'
sys.path.append(os.path.abspath(os.path.join(os.path.realpath(__file__), "..", "..")))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", SETTINGS)
django.setup()

from datetime import datetime
from search.analyzers import *
from elasticsearch_dsl import Index
from elasticsearch_dsl.connections import Elasticsearch
from elasticsearch_dsl.query import Q, FunctionScore
from elasticsearch_dsl.aggs import A
from examples.utils import print_response, pretty_json, print_prepared_query
from search.documents import CarDocument
from cars.models import Car


# 1] match all cars with brand name, which includes "ko"
#    - NOTE: it found Skoda, because we have partial analyzer on it (not edge)
#
# query = Q("match", brand_name="ko")
# search = CarDocument.search().query(query)
# response = search.execute()
# print_prepared_query(search)
# print_response(response, fields=["name", "brand_name", "score"])


# 2] match all cars, which have "ško" in tags (only czech translation)
#    - NOTE: query_2 does not found anything, because we have bad
#            analyzer to work with diacritics

# query = Q("match", tags__cs="ško")
# query_2 = Q("match", tags__cs={"query": "ško", "analyzer": "standard"})
# search = CarDocument.search().query(query)
# response = search.execute()
# print_prepared_query(search)
# print_response(response, fields=["name", "brand_name", "tags", "score"])


# 3] match with fuziness
#    - using fuzziness to correct the TYPO in query
#    - NOTE: if letters are next to each other, there more than 1 letter can missing
#            but if the distance between them is higher then 0,
#            then fuzziness = number of characters
# query = Q("match", name={"query": "Octavia", "fuzziness": 0})
# search = CarDocument.search().query(query)
# response = search.execute()
# print_prepared_query(search)
# print_response(response, fields=["name", "brand_name", "tags", "score"])

# 4] multimatch with boosting, match cars with "Černá fabia 1800" in tags or name
#   - multimatch gives use a possibility to search in multiple fields
#   - NOTE: we using boosting of single field name, because we want it at the first place
#   - NOTE: we using filter_query to have results only by "Škoda" brand
# query = Q("multi_match", query="Černá fabia 1800", fields=["name^2", "tags.cs"])
# filter_query = Q('match', tags__cs="skoda")
# search = CarDocument.search().query(query).filter(filter_query)
# response = search.execute()
# print_prepared_query(search)
# print_response(response, fields=["name", "brand_name", "tags", "score"])


# 5] function score with multimatch, boosting and fuzziness
#    - NOTE: boost - if multimatch query find match with query string, this document is boosted
#    - NOTE: functions - if filters inside find match with query string, this documents is boosted by
#            weight number
#    - NOTE: boost mode - at the end matches from top level query and functions are multiplied

# multi_match_query = FunctionScore(
#    query=Q("multi_match", query="Černý mercedes diesel automatická převodovka 2020", fields=['name', 'description', 'tags.cs']),
#    boost=2,
#    functions=[
#        {"filter": Q("match", name={"query": "Černý diesel automatická převodovka 2020", "fuzziness": 2}), "weight": 4},
#        {"filter": Q("match", tags__cs={"query": "Černý diesel automatická převodovka 2020", "fuzziness": 2}), "weight": 3},
#        {"filter": Q("match", description__cs={"query": "Černý diesel automatická převodovka 2020", "fuzziness": 2}), "weight": 2},
#    ],
#    boost_mode="multiply"
# )
#
# search = CarDocument.search().query(multi_match_query)
# response = search.execute()
# print_prepared_query(search)
# print_response(response, fields=["name", "description", "params", "brand_name", "tags", "score"])
#
#
