import os
import sys
import django

SETTINGS = 'elasticsearch_cars.settings'
sys.path.append(os.path.abspath(os.path.join(os.path.realpath(__file__), "..", "..")))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", SETTINGS)
django.setup()

from time import time
from datetime import datetime
from search.analyzers import *
from elasticsearch import helpers
from elasticsearch_dsl import Index
from elasticsearch_dsl.connections import Elasticsearch, connections
from search.documents import CarDocument
from cars.models import Car


cars = list(Car.objects.all()) * 100  # 1000 instances (we have 10 in DB)
car_document_list = []
for car in cars:
    car_document = CarDocument(
        _id=car.id,
        name=car.name,
        price=car.price,
        created_dt=car.created_dt,
        code=car.code,
        brand_name=car.brand.name
    )
    car_document.add_description(car)
    car_document.add_params(car.params.all())
    car_document.add_tags(car.params.all())

    car_document_list.append(car_document)


# 1] indexing data via .save() method
# time_start = time()
# for car in car_document_list:
#     car.save()
# time_end = time()
# print(f"elapsed = {time_end-time_start}")

# 2] bulk indexing
#    - much faster
#
# def get_data():
#
#     for car in car_document_list:
#         yield car.to_dict(include_meta=True)
#
# client = connections.get_connection()
# time_start = time()
# helpers.bulk(
#    client,
#    get_data()
# )
# time_end = time()
# print(f"elapsed = {time_end - time_start}")
