import os
import sys
import django

SETTINGS = 'elasticsearch_cars.settings'
sys.path.append(os.path.abspath(os.path.join(os.path.realpath(__file__), "..", "..")))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", SETTINGS)
django.setup()

from datetime import datetime
from search.analyzers import *
from elasticsearch_dsl import Index, Document, Text, Keyword, Double, InnerDoc, Nested, Object
from elasticsearch_dsl.connections import Elasticsearch
from cars.models import Car
from examples.utils import pretty_json


class MultilangTextInnerDoc(InnerDoc):

    cs = Text()
    en = Text()


class ParamsInnerDoc(InnerDoc):

    name = Keyword()
    value = Keyword()


class CarDocument(Document):
    name = Object(MultilangTextInnerDoc)
    code = Keyword()
    price = Double()
    params = Nested(ParamsInnerDoc)


    class Index:
        name = "elastic-cars"
        settings = {}

    @classmethod
    def init(cls, index=None, using=None):
        # adding timestamp to new index name
        # register aliases

        timestamp = datetime.now().strftime('%Y_%m_%d_%H_%M_%S')
        new_index_name = f"{cls._index._name}_{timestamp}"
        cls._index.aliases(**{"elastic-cars": {}})
        # register analyzers
        super().init(index=new_index_name, using=using)


# 1] get mapping
# CarDocument.init()
# idx = Index("elastic-cars")
# print(pretty_json(idx.get_mapping()))
#
