import sys
import json

# reconfigure stdout
sys.stdout.reconfigure(encoding='utf-8')


def print_response(response, fields=None, pretty=True):
    print()
    print('RESPONSE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
    fields_list = []
    for hit in response.hits:
        # include also meta attributes like: _score, id, index, doc_type
        fields_dict = {
            **hit.meta.to_dict(),
            **hit.to_dict()
        }

        if fields is not None:
            fields_dict = {field: value for field, value in fields_dict.items() if field in fields}

        fields_list.append(fields_dict)

    if pretty:
        fields_list = pretty_json(fields_list)

    print()
    print(f"success = {response.success()}")
    print(f"took = {response.took}")
    print(f"total_count = {response.hits.total}")
    print(f"max_score = {response.hits.max_score}")
    print()
    print(fields_list)

    print()
    print('AGGREGATIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
    if response.aggregations:

        aggs_dict = response.aggregations.to_dict()
        if fields is not None:
            aggs_dict = {agg_name: data for agg_name, data in aggs_dict.items()}

        if pretty:
            aggs_dict = pretty_json(aggs_dict)

        print(aggs_dict)


def print_prepared_query(query, pretty=True):
    print()
    print("PREPARED QUERY ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
    query = str(query.to_dict()).replace("'", '"')

    if pretty:
        print(pretty_json(query))
    else:
        print(query)


def pretty_json(data):

    try:
        data = str(data).replace("'", '"').replace("None", "null")
    except Exception as e:
        print(e)
        return

    data = json.loads(data)
    # ensure_ascii => diacritics
    return json.dumps(data, indent=2, ensure_ascii=False, sort_keys=True)


