from django.db import models
from django.db.models import SlugField
from django.utils.text import slugify
from django.conf import settings
from django.utils.translation import gettext_lazy as _

from parler.models import TranslatableModel, TranslatedFields
from parler.utils.context import switch_language


class Param(TranslatableModel):
    name_slug = SlugField(_("name slug"))
    value_slug = SlugField(_("value slug"))
    translations = TranslatedFields(
        name=models.CharField(_("name"), max_length=128),
        value=models.CharField(_("value"), max_length=128)
    )

    class Meta:
        verbose_name = _("parameter")
        verbose_name_plural = _("parameters")
        unique_together = ["name_slug", "value_slug"]

    def __str__(self):
        return f"{self.name} - {self.value}"

    def save(self, *args, **kwargs):

        # set the slug field based on default language
        with switch_language(self, settings.LANGUAGE_CODE):
            self.name_slug = slugify(self.name)
            self.value_slug = slugify(self.value)

        super().save(*args, **kwargs)
