from elasticsearch_dsl import (analyzer, tokenizer, token_filter, char_filter)


class TokenFilters:
    ngram_token_filter = token_filter(
        'ngram_token_filter', type='ngram', min_gram=2, max_gram=12
    )
    edge_ngram_token_filter = token_filter(
        'edge_ngram_token_filter', type='edge_ngram', min_gram=2, max_gram=12
    )
    czech_stop_token_filter = token_filter(
        'czech_stop_token_filter', type='stop', stopwords='_czech_'
    )
    czech_stemmer_token_filter = token_filter(
        'czech_stemmer_token_filter', type='stemmer', name='czech'
    )


class Analyzers:
    partial_word_analyzer = analyzer(
        'partial_word_analyzer',
        tokenizer='standard',
        filter=[
            'lowercase',
            'asciifolding',
            TokenFilters.ngram_token_filter,
            TokenFilters.czech_stop_token_filter
        ],
        char_filter=["html_strip"]
    )
    partial_edge_word_analyzer = analyzer(
        'partial_edge_word_analyzer',
        tokenizer='standard',
        filter=[
            'lowercase',
            'asciifolding',
            TokenFilters.czech_stop_token_filter,
            TokenFilters.edge_ngram_token_filter
        ],
        char_filter=["html_strip"]
    )

    ascii_fold = analyzer(
        'ascii_fold',
        tokenizer='standard',
        filter=[
            'lowercase',
            'asciifolding',
            TokenFilters.czech_stop_token_filter
        ],
        char_filter=["html_strip"]
    )
